@@ -7,12 +7,16 @@ class GCS::Entry {
 public:
 	Entry(int id, int gd, const QString &name, const GCS &gcs)
 	  : _id(id), _gd(gd), _name(name), _gcs(gcs) {}
+	Entry(int id) : _id(id) {}
 
 	int id() const {return _id;}
 	int gd() const {return _gd;}
 	const QString &name() const {return _name;}
 	const GCS &gcs() const {return _gcs;}
 
+	bool operator<(const Entry &other) const
+	  {return _id < other._id;}
+
 private:
 	int _id, _gd;
 	QString _name;
@@ -59,14 +63,18 @@ QList<GCS::Entry> GCS::defaults()
 
 GCS GCS::gcs(int id)
 {
-	for (int i = 0; i < _gcss.size(); i++)
-		if (_gcss.at(i).id() == id)
-			return _gcss.at(i).gcs();
+	// There are GCSs without EPSG code (id = 0) in the list!
+	if (!id)
+		return GCS();
 
-	return GCS();
+	QList<GCS::Entry>::iterator it = std::lower_bound(
+	  _gcss.begin(), _gcss.end(), id);
+
+	return (it == _gcss.end() || id != it->id()) ? GCS() : it->gcs();
 }
 
-GCS GCS::gcs(int geodeticDatum, int primeMeridian, int angularUnits)
+GCS GCS::gcs(int geodeticDatum, const PrimeMeridian &primeMeridian,
+  const AngularUnits &angularUnits)
 {
 	for (int i = 0; i < _gcss.size(); i++) {
 		const Entry &e = _gcss.at(i);
@@ -95,97 +103,97 @@ bool GCS::loadList(const QString &path)
 	bool res;
 
 	if (!file.open(QFile::ReadOnly)) {
-		qWarning("Error opening GCS file: %s: %s", qPrintable(path),
-		  qPrintable(file.errorString()));
+		qWarning("%s: %s", qUtf8Printable(path),
+		  qUtf8Printable(file.errorString()));
 		return false;
 	}
 
 	while (!csv.atEnd()) {
 		if (!csv.readEntry(entry)) {
-			qWarning("%s:%d: Parse error", qPrintable(path), csv.line());
+			qWarning("%s:%d: Parse error", qUtf8Printable(path), csv.line());
 			return false;
 		}
 		if (entry.size() < 14) {
-			qWarning("%s:%d: Invalid column count", qPrintable(path),
+			qWarning("%s:%d: Invalid column count", qUtf8Printable(path),
 			  csv.line() - 1);
 			return false;
 		}
 
 		int id = parameter(entry.at(1), &res);
 		if (!res) {
-			qWarning("%s:%d: Invalid GCS code", qPrintable(path),
+			qWarning("%s:%d: Invalid GCS code", qUtf8Printable(path),
 			  csv.line() - 1);
 			continue;
 		}
 		int gd = parameter(entry.at(2), &res);
 		if (!res) {
-			qWarning("%s:%d: Invalid geodetic datum code", qPrintable(path),
+			qWarning("%s:%d: Invalid geodetic datum code", qUtf8Printable(path),
 			  csv.line() - 1);
 			continue;
 		}
 		int au = entry.at(3).toInt(&res);
 		if (!res) {
-			qWarning("%s:%d: Invalid angular units code", qPrintable(path),
+			qWarning("%s:%d: Invalid angular units code", qUtf8Printable(path),
 			  csv.line() - 1);
 			continue;
 		}
 		int el = entry.at(4).toInt(&res);
 		if (!res) {
-			qWarning("%s:%d: Invalid ellipsoid code", qPrintable(path),
+			qWarning("%s:%d: Invalid ellipsoid code", qUtf8Printable(path),
 			  csv.line() - 1);
 			continue;
 		}
 		int pm = entry.at(5).toInt(&res);
 		if (!res) {
-			qWarning("%s:%d: Invalid prime meridian code", qPrintable(path),
+			qWarning("%s:%d: Invalid prime meridian code", qUtf8Printable(path),
 			  csv.line() - 1);
 			continue;
 		}
 		int ct = entry.at(6).toInt(&res);
 		if (!res) {
 			qWarning("%s:%d: Invalid coordinates transformation code",
-			  qPrintable(path), csv.line() - 1);
+			  qUtf8Printable(path), csv.line() - 1);
 			continue;
 		}
 		double dx = entry.at(7).toDouble(&res);
 		if (!res) {
-			qWarning("%s:%d: Invalid dx", qPrintable(path), csv.line() - 1);
+			qWarning("%s:%d: Invalid dx", qUtf8Printable(path), csv.line() - 1);
 			continue;
 		}
 		double dy = entry.at(8).toDouble(&res);
 		if (!res) {
-			qWarning("%s:%d: Invalid dy", qPrintable(path), csv.line() - 1);
+			qWarning("%s:%d: Invalid dy", qUtf8Printable(path), csv.line() - 1);
 			continue;
 		}
 		double dz = entry.at(9).toDouble(&res);
 		if (!res) {
-			qWarning("%s:%d: Invalid dz", qPrintable(path), csv.line() - 1);
+			qWarning("%s:%d: Invalid dz", qUtf8Printable(path), csv.line() - 1);
 			continue;
 		}
 		double rx = parameterd(entry.at(10), &res);
 		if (!res) {
-			qWarning("%s:%d: Invalid rx", qPrintable(path), csv.line() - 1);
+			qWarning("%s:%d: Invalid rx", qUtf8Printable(path), csv.line() - 1);
 			continue;
 		}
 		double ry = parameterd(entry.at(11), &res);
 		if (!res) {
-			qWarning("%s:%d: Invalid ry", qPrintable(path), csv.line() - 1);
+			qWarning("%s:%d: Invalid ry", qUtf8Printable(path), csv.line() - 1);
 			continue;
 		}
 		double rz = parameterd(entry.at(12), &res);
 		if (!res) {
-			qWarning("%s:%d: Invalid rz", qPrintable(path), csv.line() - 1);
+			qWarning("%s:%d: Invalid rz", qUtf8Printable(path), csv.line() - 1);
 			continue;
 		}
 		double ds = parameterd(entry.at(13), &res);
 		if (!res) {
-			qWarning("%s:%d: Invalid ds", qPrintable(path), csv.line() - 1);
+			qWarning("%s:%d: Invalid ds", qUtf8Printable(path), csv.line() - 1);
 			continue;
 		}
 
 		const Ellipsoid &e = Ellipsoid::ellipsoid(el);
 		if (e.isNull()) {
-			qWarning("%s:%d: Unknown ellipsoid code", qPrintable(path),
+			qWarning("%s:%d: Unknown ellipsoid code", qUtf8Printable(path),
 			  csv.line() - 1);
 			continue;
 		}
@@ -203,12 +211,12 @@ bool GCS::loadList(const QString &path)
 				break;
 			default:
 				qWarning("%s:%d: Unknown coordinates transformation method",
-				  qPrintable(path), csv.line() - 1);
+				  qUtf8Printable(path), csv.line() - 1);
 				continue;
 		}
 		if (!datum.isValid()) {
 			qWarning("%s:%d: Invalid coordinates transformation parameters",
-			  qPrintable(path), csv.line() - 1);
+			  qUtf8Printable(path), csv.line() - 1);
 			continue;
 		}
 
@@ -217,9 +225,11 @@ bool GCS::loadList(const QString &path)
 			_gcss.append(Entry(id, gd, entry.at(0), gcs));
 		else
 			qWarning("%s:%d: Unknown prime meridian/angular units code",
-			  qPrintable(path), csv.line() - 1);
+			  qUtf8Printable(path), csv.line() - 1);
 	}
 
+	std::sort(_gcss.begin(), _gcss.end());
+
 	return true;
 }
 
@@ -229,7 +239,7 @@ QList<KV<int, QString> > GCS::list()
 
 	for (int i = 0; i < _gcss.size(); i++) {
 		const Entry &e = _gcss.at(i);
-		if (!e.id() || (i && e.id() == list.last().key()))
+		if (!e.id() || (!list.isEmpty() && e.id() == list.last().key()))
 			continue;
 
 		list.append(KV<int, QString>(e.id(), e.name() + " / Geographic 2D"));
